__all__ = [
    "__name__",
    "__description__",
    "__version__",
    "__url__",
    "__license__",
    "__author__",
    "__author_email__",
]

__name__ = "pyimagetest"
__description__ = "Utilities for unit testing with images"
__version__ = "0.2.post0"
__url__ = "https://github.com/pmeier/pyimagetest"
__license__ = "BSD-3"
__author__ = "Philip Meier"
__author_email__ = "github.pmeier@posteo.de"
